import { Component, OnInit, OnDestroy, ViewChild, AfterViewInit } from '@angular/core';
import { OrdersApiService } from 'app/core/services/orders.service';
import { MatDialog } from '@angular/material/dialog';
import { Orders } from 'app/shared/models/orders';
import { OrdersDialogComponent } from './orders-dialog/orders-dialog.component';
import { ConfirmDialogModel } from 'app/shared/models/confirm-dialog';
import { ConfirmDialogComponent } from 'app/shared/components/confirm-dialog/confirm-dialog.component';
import { NotificationsService } from 'app/core/services/notifications.service';
import { Subject } from 'rxjs';
import { DataTableDirective } from 'angular-datatables';

@Component({
    selector: 'app-Orders-cmp',
    templateUrl: 'Orders.component.html',
    providers: [OrdersApiService]
})

export class OrdersComponent implements OnInit, OnDestroy {
    public dtos: Orders[];

    @ViewChild(DataTableDirective,  {static: false}) dtElement: DataTableDirective;
    public dtTrigger: Subject<any> = new Subject();
    public dtOptions: any;

    constructor(
        public dialog: MatDialog,
        private ordersApiService: OrdersApiService,
        private notificationsService: NotificationsService) { }

    ngOnInit() {
        this.dtOptions = {
            pagingType: 'full_numbers',
            lengthMenu: [
                [5, 10, 20, -1],
                [5, 10, 20, 'All']
            ],
            language: {
                search: '_INPUT_',
                searchPlaceholder: 'Search records',
                emptyTable: 'Fetching Orders...'
            },
            scrollX: true,
            responsive: true,
            // Declare the use of the extension in the dom parameter
            dom: 'Bfrtip',
            
            buttons: [
                
                'copy',
                'print',
                'excel',
                
            ]
        };

        this.getAllOrderss();
    }

    ngAfterViewInit() {
        this.renderDataTable();
    }

    getAllOrderss() {
        this.ordersApiService.getAllOrderss().subscribe(dtos => {
            this.dtos = dtos;
            if (this.dtos.length == 0) {
                this.dtOptions.language.emptyTable = 'No dtos available yet.';
            }

            this.renderDataTable();
        }, (err) => {
            this.renderDataTable();
            this.dtOptions.language.emptyTable = 'Error while fetching dtos, pls try refreshing the page or contact admin.';
            this.notificationsService.showNotification('Error', err.message, 'warning');
        });
    }

    addDialog() {
        const dialogRef = this.dialog.open(OrdersDialogComponent, {
            width: '650px',
            data: new Orders()
        });

        dialogRef.afterClosed().subscribe(orders => {
            if (orders) {
                this.ordersApiService.addOrders(orders).subscribe(() => {
                    this.getAllOrderss();
                    this.notificationsService.showNotification('Success', 'Orders added successfully', 'success');
                }, (err) => {
                    this.notificationsService.showNotification('Error', err.message, 'warning');
                });
            }
        });
    }

    editDialog(orders: Orders) {
        const dialogRef = this.dialog.open(OrdersDialogComponent, {
            width: '650px',
            data: orders
        });

        dialogRef.afterClosed().subscribe(orders => {
            if (orders) {
                this.ordersApiService.updateOrders(orders).subscribe(() => {
                    this.getAllOrderss();
                    this.notificationsService.showNotification('Success', 'Orders updated successfully', 'success');
                }, (err) => {
                    this.notificationsService.showNotification('Error', err.message, 'warning');
                });
            }
        });
    }

    deleteDialog(orders: Orders) {
        const dialogData = new ConfirmDialogModel("Really delete?", orders._id.toString());
        const dialogRef = this.dialog.open(ConfirmDialogComponent, {
            width: "400px",
            data: dialogData
        });

        dialogRef.afterClosed().subscribe(remove => {
            if (remove) {
                this.ordersApiService.deleteOrders(orders._id).subscribe(() => {
                    this.getAllOrderss();
                    this.notificationsService.showNotification('Success', 'Orders deleted successfully', 'success');
                }, (err) => {
                    this.notificationsService.showNotification('Error', err.message, 'warning');
                });
            }
        });
    }

    renderDataTable(): void {
        if (this.dtElement && this.dtElement.dtInstance) {
            this.dtElement.dtInstance.then((dtInstance: DataTables.Api) => {
                dtInstance.destroy();
            });
        }

        this.dtTrigger.next();
    }

    ngOnDestroy(): void {
        this.dtTrigger.unsubscribe();
    }
}

